#include <test_harness.hpp>

/**
 * Functor for matrix productions of two variable matrices.
 */
struct matrix_product_eigen_fun {
  template <typename T>
  T operator()(const Eigen::Matrix<T, Eigen::Dynamic, 1>& x) const {
    using Eigen::Matrix;
    using Eigen::Dynamic;
    int N = static_cast<int>(std::sqrt(x.size() / 2));
    Matrix<T, Dynamic, Dynamic> a(N,N);
    Matrix<T, Dynamic, Dynamic> b(N,N);
    int i = 0;
    for (int m = 0; m < N; ++m) {
      for (int n = 0; n < N; ++n) {
        a(m,n) = x(i++);
        b(m,n) = x(i++);
      }
    }
    Matrix<T, Eigen::Dynamic, Eigen::Dynamic> ab = a * b; // force assignment
    return ab(0,0);
  }

  static void fill(Eigen::VectorXd& x) {
    int N = static_cast<int>(sqrt(x.size() / 2));
    if (N < 1) N = 1;
    x.resize(N * N * 2);
    for (int i = 0; i < x.size(); ++i)
      x(i) = static_cast<double>(i + 1) / (x.size() + 1);
  }

  static std::string name() {
    return "matrix_product_eigen_nosum";
  }
};

int main() {
  run_test<matrix_product_eigen_fun>();
  return 0;
}
